@@ -7,8 +7,14 @@ use Test::Run::Straps;
 use Test::Run::Output;
 use Test::Run::Base;
 use Test::Run::Assert;
-use Test::Run::Obj::Structs;
+
+use Test::Run::Obj::FailedObj;
+use Test::Run::Obj::TestObj;
+use Test::Run::Obj::TotObj;
+use Test::Run::Obj::CanonFailedObj;
+
 use Test::Run::Obj::Error;
+
 use Exporter;
 use Benchmark;
 use Config;
@@ -17,7 +23,7 @@ use strict;
 use Class::Accessor;
 
 use Scalar::Util ();
-
+use List::Util qw(max);
 use vars qw(
     $VERSION
     @ISA
@@ -35,11 +41,11 @@ Test::Run::Core - Run Perl standard test scripts with statistics
 
 =head1 VERSION
 
-Version 0.0105
+Version 0.0106
 
 =cut
 
-$VERSION = '0.0105';
+$VERSION = '0.0106';
 
 $ENV{HARNESS_ACTIVE} = 1;
 $ENV{HARNESS_NG_VERSION} = $VERSION;
@@ -56,22 +62,25 @@ __PACKAGE__->mk_accessors(qw(
     _bonusmsg
     Columns
     Debug
-    Leaked_Dir
-    NoTty
-    Strap
-    Switches
-    Switches_Env
-    Test_Interpreter
-    Timer
-    Verbose
     dir_files
     failed_tests
     format_columns
+    last_test_elapsed
+    last_test_obj
+    last_test_results
+    Leaked_Dir
     list_len
     max_namelen
+    NoTty
     output
+    Strap
+    Switches
+    Switches_Env
     test_files
+    Test_Interpreter
+    Timer
     tot
+    Verbose
     width
 ));
 
@@ -537,13 +546,15 @@ sub _recheck_dir_files
 sub _tot_add
 {
     my ($self, $field, $diff) = @_;
+
     $self->tot()->add($field, $diff);
 }
 
 sub _tot_inc
 {
     my ($self, $field) = @_;
-    $self->_tot_add($field,1);
+
+    $self->tot()->inc($field);
 }
 
 sub _create_failed_obj_instance
@@ -557,16 +568,17 @@ sub _create_failed_obj_instance
 
 sub _is_failed_and_max
 {
-    my ($self, $args) = @_;
-    my $test = $args->{'test_struct'};
+    my ($self) = @_;
+
+    my $test = $self->last_test_obj;
 
     return (@{$test->failed()} and $test->max());
 }
 
 sub _get_failed_and_max_msg
 {
-    my ($self, $args) = @_;
-    my $test = $args->{'test_struct'};
+    my ($self) = @_;
+    my $test = $self->last_test_obj;
 
     my ($txt) = $self->_canonfailed($test);
 
@@ -576,7 +588,7 @@ sub _get_failed_and_max_msg
 sub _get_dont_know_which_tests_failed_msg
 {
     my ($self, $args) = @_;
-    my $test = $args->{'test_struct'};
+    my $test = $self->last_test_obj;
     
     return
         ("Don't know which tests failed: got " . $test->ok() . " ok, ".
@@ -586,13 +598,13 @@ sub _get_dont_know_which_tests_failed_msg
 
 sub _get_failed_with_results_seen_msg
 {
-    my ($self, $args) = @_;
-    my $test = $args->{'test_struct'};
+    my ($self) = @_;
+    my $test = $self->last_test_obj;
     
     return 
-        $self->_is_failed_and_max($args)
-            ? $self->_get_failed_and_max_msg($args)
-            : $self->_get_dont_know_which_tests_failed_msg($args)
+        $self->_is_failed_and_max()
+            ? $self->_get_failed_and_max_msg()
+            : $self->_get_dont_know_which_tests_failed_msg()
             ;
 }
 
@@ -632,9 +644,9 @@ The results of the test.
 
 sub _get_failed_and_max_params
 {
-    my ($self, $args) = @_;
+    my ($self) = @_;
     
-    my $test = $args->{'test_struct'};
+    my $test = $self->last_test_obj;
 
     my (undef, $canon) = $self->_canonfailed($test);
 
@@ -660,14 +672,12 @@ sub _get_undef_tests_params
 
 sub get_common_FWRS_params
 {
-    my ($self, $args) = @_;
-
-    my $test = $args->{'test_struct'};
+    my ($self) = @_;
 
     return
         [
-            max     => $test->max(),
-            name    => $args->{'filename'},
+            max     => $self->last_test_obj->max(),
+            name    => $self->_get_last_test_filename(),
             estat   => '',
             wstat   => '',
         ];
@@ -675,38 +685,38 @@ sub get_common_FWRS_params
 
 sub _get_FWRS_tests_existence_params
 {
-    my ($self, $args) = @_;
+    my ($self) = @_;
 
     return
         [
-            $self->_is_failed_and_max($args)
-            ? (@{$self->_get_failed_and_max_params($args)})
-            : (@{$self->_get_undef_tests_params($args)})
+            $self->_is_failed_and_max()
+            ? (@{$self->_get_failed_and_max_params()})
+            : (@{$self->_get_undef_tests_params()})
         ]
 }
 
 sub _get_failed_with_results_seen_params
 {
-    my ($self, $args) = @_;
+    my ($self) = @_;
 
     return 
         {
-            @{$self->get_common_FWRS_params($args)},
-            @{$self->_get_FWRS_tests_existence_params($args)},
+            @{$self->get_common_FWRS_params()},
+            @{$self->_get_FWRS_tests_existence_params()},
         }
 }
 
 sub _failed_with_results_seen
 {
-    my ($self, $args) = @_;
+    my ($self) = @_;
 
     $self->_tot_inc('bad');
 
-    $self->_report_failed_with_results_seen($args);
+    $self->_report_failed_with_results_seen();
 
     return
         $self->_create_failed_obj_instance(
-            $self->_get_failed_with_results_seen_params($args)
+            $self->_get_failed_with_results_seen_params()
         );
 }
 
@@ -718,9 +728,7 @@ sub _failed_with_results_seen
 
 sub _failed_before_any_test_output
 {
-    my ($self, $args) = @_;
-
-    my $tfile = $args->{'filename'};
+    my ($self) = @_;
 
     $self->_report_failed_before_any_test_output();
 
@@ -731,7 +739,7 @@ sub _failed_before_any_test_output
             canon       => '??',
             max         => '??',
             failed      => '??',
-            name        => $tfile,
+            name        => $self->_get_last_test_filename(),
             percent     => undef,
             estat       => '', 
             wstat       => '',
@@ -739,36 +747,59 @@ sub _failed_before_any_test_output
         );
 }
 
+=for Hello
+            {
+                test_struct => $self->last_test_obj(),
+                estatus => $results->{exit},
+                wstatus => $results->{wait},
+                filename => $test_file,
+                results => $results,
+            }
+=cut
+
+sub _get_wstatus
+{
+    my $self = shift;
+
+    return $self->last_test_results()->{'wait'};
+}
+
+sub _get_estatus
+{
+    my $self = shift;
+
+    return $self->last_test_results()->{'exit'};
+}
+
+sub _is_last_test_seen
+{
+    return shift->last_test_results->seen;
+}
+
 sub _get_failed_struct
 {
-    my ($self, $args) = @_;
-    if ($args->{'wstatus'}) {
-         return
-            $self->_dubious_return(
-                $args
-                );
+    my ($self) = @_;
+
+    if ($self->_get_wstatus())
+    {
+         return $self->_dubious_return();
     }
-    elsif($args->{'results'}->seen()) {
-        return
-            $self->_failed_with_results_seen(
-                $args,
-            );
+    elsif($self->_is_last_test_seen())
+    {
+        return $self->_failed_with_results_seen();
     }
-    else {
-        return
-            $self->_failed_before_any_test_output(
-                $args,
-            );
+    else
+    {
+        return $self->_failed_before_any_test_output();
     }
 }
 
 sub _list_tests_as_failures
 {
     my $self = shift;
-    my $args = shift;
 
-    my $test = $args->{test_struct};
-    my $results = $args->{results};
+    my $test = $self->last_test_obj;
+    my $results = $self->last_test_results;
 
     # List unrun tests as failures.
     if ($test->next() <= $test->max()) {
@@ -852,7 +883,7 @@ sub _time_single_test
     return ($results, $elapsed);
 }
 
-=head2 $self->_report_skipped_test({test_struct => $test, elapsed => $elapsed})
+=head2 $self->_report_skipped_test()
 
 [This is a method that needs to be over-rided.]
 
@@ -862,12 +893,12 @@ Should report the skipped test.
 
 sub _process_skipped_test
 {
-    my ($self, $args) = @_;
+    my ($self) = @_;
 
-    return $self->_report_skipped_test($args);
+    return $self->_report_skipped_test();
 }
 
-=head2 $self->_report_all_ok_test({test_struct => $test, elapsed => $elapsed})
+=head2 $self->_report_all_ok_test()
 
 [This is a method that needs to be over-rided.]
 
@@ -877,11 +908,11 @@ Should report the all OK test.
 
 sub _process_all_ok_test
 {
-    my ($self, $args) = @_;
-    return $self->_report_all_ok_test($args);
+    my ($self) = @_;
+    return $self->_report_all_ok_test();
 }
 
-=head2 $self->_report_all_skipped_test({test_struct => $test, elapsed => $elapsed})
+=head2 $self->_report_all_skipped_test()
 
 [This is a method that needs to be over-rided.]
 
@@ -891,31 +922,31 @@ Should report the all-skipped test.
 
 sub _process_all_skipped_test
 {
-    my ($self, $args) = @_;
+    my ($self) = @_;
 
-    $self->_report_all_skipped_test($args);
+    $self->_report_all_skipped_test();
     $self->_tot_inc('skipped');
 }
 
 sub _process_passing_test
 {
-    my ($self, $args) = @_;
+    my ($self) = @_;
 
-    my $test = $args->{test_struct};
-    my $elapsed = $args->{elapsed};
+    my $test = $self->last_test_obj;
+    my $elapsed = $self->last_test_elapsed;
 
     # XXX Combine these first two
     if ($test->max() and $test->skipped() + $test->bonus())
     {
-        $self->_process_skipped_test($args);
+        $self->_process_skipped_test();
     }
     elsif ( $test->max() )
     {
-        $self->_process_all_ok_test($args);        
+        $self->_process_all_ok_test();
     }
     else
     {
-        $self->_process_all_skipped_test($args);
+        $self->_process_all_skipped_test();
     }
     $self->_tot_inc('good');
 }
@@ -937,14 +968,14 @@ sub _calc_test_struct_ml
     return "";
 }
 
-sub _get_test_struct
+sub _calc_test_struct
 {
     my $self = shift;
-    my $results = shift;
+    my $results = $self->last_test_results;
 
     $self->_tot_add_results($results);
 
-    return 
+    return $self->last_test_obj(
         $self->_create_test_obj_instance(
             {
                 ok          => $results->ok(),
@@ -961,7 +992,8 @@ sub _get_test_struct
                 skip_all    => $results->skip_all(),
                 ml          => $self->_calc_test_struct_ml($results),
             }
-        );
+        )
+    );
 }
 
 =head2 $self->_report_single_test_file_start({test_file => "t/my_test_file.t"})
@@ -985,6 +1017,49 @@ sub _prepare_for_single_test_run
     return;
 }
 
+sub _get_last_test_filename
+{
+    my $self = shift;
+
+    return $self->last_test_results->filename;
+}
+
+sub _add_to_failed_tests
+{
+    my $self = shift;
+
+    $self->failed_tests()->{$self->_get_last_test_filename()} = 
+        $self->_get_failed_struct();
+
+    return;
+}
+
+sub _is_test_passing
+{
+    my $self = shift;
+
+    return $self->last_test_results->{passing};
+}
+
+sub _process_test_file_results
+{
+    my ($self) = @_;
+
+    $self->_calc_test_struct();
+
+    if ($self->_is_test_passing()) 
+    {
+        $self->_process_passing_test();
+    }
+    else
+    {
+        $self->_list_tests_as_failures();
+        $self->_add_to_failed_tests();
+    }
+
+    return;
+}
+
 sub _run_single_test
 {
     my ($self, $args) = @_;
@@ -995,37 +1070,10 @@ sub _run_single_test
 
     my ($results, $elapsed) = $self->_time_single_test($tfile);
 
-    my $test = $self->_get_test_struct($results);
+    $self->last_test_results($results);
+    $self->last_test_elapsed($elapsed);
 
-    my($estatus, $wstatus) = @{$results}{qw(exit wait)};
-
-    if ($results->{passing}) 
-    {
-        $self->_process_passing_test(
-            {
-                test_struct => $test,
-                elapsed => $elapsed,
-            },
-        );
-    }
-    else {
-        $self->_list_tests_as_failures(
-            {
-                'test_struct' => $test,
-                'results' => $results,
-            }
-        ); 
-        $self->failed_tests()->{$tfile} = 
-            $self->_get_failed_struct(
-                {
-                    test_struct => $test,
-                    estatus => $estatus,
-                    wstatus => $wstatus,
-                    filename => $tfile,
-                    results => $results,
-                }
-            );
-    }
+    $self->_process_test_file_results();
 
     $self->_recheck_dir_files();
 }
@@ -1088,14 +1136,20 @@ longest test name.
 
 =cut
 
-sub _leader_width {
+sub _max_len
+{
+    my ($self, $array_ref) = @_;
+
+    return max(map { length($_) } @$array_ref);
+}
+
+sub _leader_width
+{
     my ($self) = @_;
     my $tests = $self->test_files();
 
-    my $maxlen = 
-        $self->__max_num_flat(map {length($_)} @$tests);
-    my $maxsuflen =
-        $self->__max_num_flat(map {length(/\.(\w+)$/ ? $1 : '')} @$tests);
+    my $maxlen =    $self->_max_len($tests);
+    my $maxsuflen = $self->_max_len([map { /\.(\w+)\z/ ? $1 : '' } @$tests]);
 
     # + 3 : we want three dots between the test name and the "ok"
     return $maxlen + 3 - $maxsuflen;
@@ -1226,23 +1280,6 @@ sub _get_format_failed_str_len
     return length($self->_get_format_failed_str());
 }
 
-sub __max_num_flat
-{
-    my $self = shift;
-    my $n = shift;
-    return $self->__max_num($n, [@_]);
-}
-
-sub __max_num
-{
-    my ($self, $max, $others )= @_;
-    foreach my $n (@$others)
-    {
-        $max = $n if $n > $max;
-    }
-    return $max;
-}
-
 sub _get_format_tests_namelens
 {
     my $self = shift;
@@ -1253,10 +1290,10 @@ sub _get_initial_max_namelen
 {
     my $self = shift;
     # Figure out our longest name string for formatting purposes.
-    return 
-        $self->__max_num(
+    return
+        max(
             $self->_get_format_failed_str_len(),
-            $self->_get_format_tests_namelens(),
+            @{$self->_get_format_tests_namelens()},
         );
 }
 
@@ -1323,25 +1360,34 @@ Prints the header of the files that failed.
 
 =cut
 
+sub _fail_other_get_canon_strings_concat
+{
+    my ($self, $ret, $first) = @_;
+
+    my $new_last_ret = "$ret->[-1] $first";
+
+    if (length($new_last_ret) < $self->list_len())
+    {
+        $ret->[-1] = $new_last_ret;
+    }
+    else
+    {
+        push @$ret, $first;
+    }
+}
+
 sub _fail_other_get_canon_strings
 {
     my $self = shift;
     my $canon = shift;
-    my @ret = ();
-    my $string = shift(@$canon);
+
+    my @ret = (shift(@$canon));
+
     while (@$canon)
     {
-        if (length($canon->[0]) + 1 + length($string)< $self->list_len())
-        {
-            $string .= " ".shift(@$canon);
-        }
-        else
-        {
-            push @ret, $string;
-            $string = shift(@$canon);
-        }
+        $self->_fail_other_get_canon_strings_concat(\@ret, shift(@$canon));
     }
-    push @ret, $string;
+
     return \@ret;
 }
 
@@ -1440,8 +1486,10 @@ sub _fail_other
     }
 }
 
-sub _show_results {
-    my($self) = @_;
+sub _show_success_or_failure
+{
+    my $self = shift;
+
     my $tot = $self->tot();
 
     if ($self->_all_ok())
@@ -1459,7 +1507,15 @@ sub _show_results {
     else
     {
         $self->_fail_other();
-    }
+    }    
+}
+
+sub _show_results
+{
+    my($self) = @_;
+
+    $self->_show_success_or_failure();
+
     $self->_report_final_stats();
 }
 
@@ -1700,15 +1756,15 @@ $args are the test-context - see above.
 
 sub _get_premature_test_dubious_summary
 {
-    my ($self, $args) = @_;
+    my ($self) = @_;
 
-    my $test = $args->{'test_struct'};
+    my $test = $self->last_test_obj;
 
     $test->add_to_failed($test->next()..$test->max());
 
     my (undef, $canon) = $self->_canonfailed($test);
 
-    $self->_report_premature_test_dubious_summary($args);
+    $self->_report_premature_test_dubious_summary();
 
     return 
     {
@@ -1722,13 +1778,13 @@ sub _get_dubious_summary
 {
     my ($self, $args) = @_;
 
-    my $test = $args->{'test_struct'};
+    my $test = $self->last_test_obj;
 
     if ($test->max())
     {
         if ($test->next() == $test->max() + 1 and not @{$test->failed()})
         {
-            $self->_report_dubious_summary_all_subtests_successful($args);
+            $self->_report_dubious_summary_all_subtests_successful();
             
             return
             {
@@ -1740,7 +1796,7 @@ sub _get_dubious_summary
         else
         {
             return
-                $self->_get_premature_test_dubious_summary($args);
+                $self->_get_premature_test_dubious_summary();
         }
     }
     else
@@ -1758,18 +1814,18 @@ sub _get_dubious_summary
 # Test program go boom.
 sub _dubious_return 
 {
-    my ($self, $args) = @_;
+    my ($self) = @_;
     
-    my $test = $args->{'test_struct'};
-    my $estatus = $args->{'estatus'};
-    my $wstatus = $args->{'wstatus'};
-    my $filename = $args->{'filename'};
+    my $test = $self->last_test_obj;
+    my $estatus = $self->_get_estatus();
+    my $wstatus = $self->_get_wstatus();
+    my $filename = $self->_get_last_test_filename();
     
-    $self->_report_dubious($args);
+    $self->_report_dubious();
 
     $self->_tot_inc('bad');
 
-    my $dubious_summary = $self->_get_dubious_summary($args);
+    my $dubious_summary = $self->_get_dubious_summary();
 
     return 
         $self->_create_failed_obj_instance(
